-- ******************************** Constants *******************************
TITAN_VECTOR_ID = "Vector"
TITAN_VECTOR_VERSION = "1.0.13.40200";
TITAN_VECTOR_BUTTON_ICON = "Interface\\Icons\\Ability_Rogue_Sprint.blp";
TITAN_VECTOR_FREQUENCY = 0.5;

-- *********************************  Strings *******************************
-- All string are now fully localised
local AceLocale = LibStub("AceLocale-3.0");
local L = AceLocale:GetLocale("TitanVector");

TITAN_VECTOR_BASE_WORD = L["TITAN_VECTOR_BASE_WORD"];
TITAN_VECTOR_TOOLTIP_TEXT =	L["TITAN_VECTOR_TOOLTIP_TEXT"];	
TITAN_VECTOR_UP_INDICATOR =	L["TITAN_VECTOR_UP_INDICATOR"];
TITAN_VECTOR_DOWN_INDICATOR =	L["TITAN_VECTOR_DOWN_INDICATOR"];
TITAN_VECTOR_HORIZONTAL_INDICATOR =	L["TITAN_VECTOR_HORIZONTAL_INDICATOR"];
TITAN_VECTOR_SHOW_LABEL_TEXT = L["TITAN_VECTOR_SHOW_LABEL_TEXT"];

-- ******************************** Variables *******************************
local TimeSlice, VectorSpeed, BaseSpeed = 0, 0, 0;
local RoundedPitch;
local AceTimer = LibStub("AceTimer-3.0");
local VectorTimer = nil;

-- **************************************************************************
-- NAME : TitanPanelVectorButton_OnLoad()
-- DESC : Registers the mod when it loads
-- **************************************************************************
function TitanPanelVectorButton_OnLoad(self)
	self.registry = {
		id = TITAN_VECTOR_ID,
		version = TITAN_VECTOR_VERSION,
		menuText = TITAN_VECTOR_ID,
		tooltipTitle = TITAN_VECTOR_ID.." "..TITAN_VECTOR_VERSION,
		tooltipTextFunction = "TitanPanelVectorButton_GetToolTipText",
		buttonTextFunction = "TitanPanelVectorButton_GetButtonText",
		category = "Information",		
		icon = TITAN_VECTOR_BUTTON_ICON,
		iconWidth = 16,
		savedVariables = {
			ShowIcon = 1,
			ShowColoredText = 1,
			ShowLabelText = 1
		},
	};
end

-- *******************************************************************************************
-- Name: TitanPanelVectorButton_GetToolTipText()
-- Desc: Gets our tool-tip text, what appears when we hover over our item on the Titan bar.
-- *******************************************************************************************
function TitanPanelVectorButton_GetToolTipText()
	return TITAN_VECTOR_TOOLTIP_TEXT;
end

-- *******************************************************************************************
-- Name: TitanPanelVectorButton_GetButtonText()
-- Desc: Gets our button text, what appears on our button on the Titan bar.
-- *******************************************************************************************
function TitanPanelVectorButton_GetButtonText()
	TITAN_VECTOR_BUTTON_TEXT = floor(BaseSpeed + 0.5).."%";
	if ( IsFlying() ) then
		RoundedPitch = format("%.2f",  GetUnitPitch("player") * 180 / math.pi);
		VectorSpeed = math.cos(GetUnitPitch("player")) * BaseSpeed;
		TITAN_VECTOR_BUTTON_TEXT = floor(VectorSpeed + 0.5).."/"..TITAN_VECTOR_BUTTON_TEXT;
		if ( tonumber(RoundedPitch) == 0.00 ) then
			TITAN_VECTOR_BUTTON_TEXT = TITAN_VECTOR_HORIZONTAL_INDICATOR..RoundedPitch.."\194\176".." - "..TITAN_VECTOR_BUTTON_TEXT;		
		elseif ( tonumber(RoundedPitch) < 0 ) then
			TITAN_VECTOR_BUTTON_TEXT = TITAN_VECTOR_DOWN_INDICATOR..(RoundedPitch * -1).."\194\176".." - "..TITAN_VECTOR_BUTTON_TEXT;
	  else
	  	TITAN_VECTOR_BUTTON_TEXT = TITAN_VECTOR_UP_INDICATOR..RoundedPitch.."\194\176".." - "..TITAN_VECTOR_BUTTON_TEXT;
	  end
	end
	-- Set the TITAN_VECTOR_BUTTON_TEXT based on the ShowColoredText settings
	if (TitanGetVar(TITAN_VECTOR_ID, "ShowColoredText")) then
		TITAN_VECTOR_BUTTON_TEXT = TitanUtils_GetGreenText(TITAN_VECTOR_BUTTON_TEXT);
  end
  if (TitanGetVar(TITAN_VECTOR_ID, "ShowLabelText")) then
  	return TITAN_VECTOR_BASE_WORD..": "..TITAN_VECTOR_BUTTON_TEXT;
  else
  	return TITAN_VECTOR_BUTTON_TEXT;
  end
end

-- *******************************************************************************************
-- Name: TitanPanelRightClickMenu_PrepareVectorMenu
-- Desc: Builds the right click config menu
-- *******************************************************************************************
function TitanPanelRightClickMenu_PrepareVectorMenu()
	if ( UIDROPDOWNMENU_MENU_LEVEL == 1 ) then
		TitanPanelRightClickMenu_AddTitle(TitanPlugins[TITAN_VECTOR_ID].menuText);
		-- Using the line below for now
		-- TitanPanelRightClickMenu_AddTitle(TITAN_VECTOR_ID);
		TitanPanelRightClickMenu_AddSpacer();
		TitanPanelRightClickMenu_AddToggleIcon(TITAN_VECTOR_ID);
		TitanPanelRightClickMenu_AddToggleColoredText(TITAN_VECTOR_ID);
		TitanPanelRightClickMenu_AddToggleVar(TITAN_VECTOR_SHOW_LABEL_TEXT, TITAN_VECTOR_ID, "ShowLabelText");
		TitanPanelRightClickMenu_AddSpacer();
		TitanPanelRightClickMenu_AddCommand(TITAN_PANEL_MENU_HIDE, TITAN_VECTOR_ID, TITAN_PANEL_MENU_FUNC_HIDE);
	end
end

-- *******************************************************************************************
-- Name: TitanPanelVectorButton_OnShow()
-- Desc: Function to create a timer object when the button is shown
-- *******************************************************************************************
function TitanPanelVectorButton_OnShow()
 -- If no AceTimer, create one
 if not VectorTimer then
   VectorTimer = AceTimer.ScheduleRepeatingTimer("TitanPanel"..TITAN_VECTOR_ID, TitanPanelPluginHandle_OnUpdate, TITAN_VECTOR_FREQUENCY, {TITAN_VECTOR_ID, TITAN_PANEL_UPDATE_BUTTON });
 end
end
-- *******************************************************************************************
-- Name: TitanPanelVectorButton_OnHide()
-- Desc: Function to create a timer object when the button is hidden
-- *******************************************************************************************
function TitanPanelVectorButton_OnHide()
  -- Clear the AceTime 
  AceTimer.CancelTimer("TitanPanel"..TITAN_VECTOR_ID, VectorTimer, true);
  VectorTimer = nil;
end

-- *******************************************************************************************
-- Name: TitanPanelVectorButton_OnUpdate(self, elapsed)
-- Desc: Function to react to the Global OnUpdate call for each frame and calculate the movement
-- *******************************************************************************************
function TitanPanelVectorButton_OnUpdate(self, elapsed)
	TimeSlice = TimeSlice + elapsed;
	-- If the TITAN_VECTOR_FREQUENCY timer hasn't fired yet then return
	if ( TimeSlice < TITAN_VECTOR_FREQUENCY ) then 
		return;
	else
		BaseSpeed = (GetUnitSpeed("Player") / 7) * 100;		
	end
	TimeSlice = 0
	TitanPanelButton_UpdateButton(TITAN_VECTOR_ID);
end

-- *******************************************************************************************
-- Name: LocalUIDropDownMenu_AddButton
-- Desc: Performs a default TitanPanel UIDropDownMenu_AddButton with parameters
-- *******************************************************************************************
function LocalUIDropDownMenu_AddButton(text, value, funcname, checked, keepshown, hasarrow, level)
	MenuItem = {};
  MenuItem.text = text;
  MenuItem.value = value;
  MenuItem.func = funcname;
  MenuItem.checked = checked;
  MenuItem.keepShownOnClick = keepshown;
  MenuItem.hasArrow = hasarrow;
	UIDropDownMenu_AddButton(MenuItem, level);
end

-- *******************************************************************************************
-- Name: TitanVectorDebug(text)
-- Desc: Outputs the text to wherever the debug is set to
-- *******************************************************************************************
function TitanVectorDebug(text)
	-- Default output to chat window
	DEFAULT_CHAT_FRAME:AddMessage("TitanVector: Debug - "..text);
	-- Use the error frame
	-- UIErrorsFrame:AddMessage("TitanVector: Debug - "..text);
end